#!/usr/bin/env python
# coding=utf-8
#    print """Modify html file to mod-html for importing to MS-Word > %s""" % (modfile)
# 
import sys
import os
import commands
import uuid
import shutil
import distutils.core
import datetime

# GLOBAL VARS
orig_toc_brk_list = [    #[TAG,LEVEL]
    ['<h1',1],
    ['<h2',2],
    ['<h3',3],
    ['<h4',4],
    ['<h5',5],
    ['<h6',6],
    ['<p class="h1"',1],
    ['<p class="ph1"',1],
    ['<div class="h1"',1],
    ['<p',6],
]
toc_brk_list = orig_toc_brk_list[:] #WILL REDUCE TO LEVEL<=max_toc_brk_list
max_toc_level = 3   #MAX TOC NEST LEVEL: DEFAULT = <h3>

epub_ver_list = ['2.0','2.0.1','3.0']
#epub_ver = '2.0'    #OK: DEPRECATED FOR GOOGLE BOOKS
epub_ver = '2.0.1'  #OK: SET AS DEFAULT VERSION
                    #HAVE MODIFIED TOC LIST INDENT IN MISS STEP NEST TOC LIST FOR GOOGLE PLAY BOOKS
#epub_ver = '3.0'    #SOME ISSUE: GOOGLE PLAY BOOKS DOES NOT SHOW NEST TOC LIST INDENT CORRECTLY
#                    # WAIT FOR NEXT GOOGLE BOOKS VERSION TO SET AS DEFAULT 

book_uuid = str(uuid.uuid4())   #UUID BOOK IDENTIFIER
book_isbn = ''                  #ISBN BOOK IDENTIFIER
coverimg = ''       #COVER IMAGE FILE

# GLABAL INTERNAL USED
progname = ''
ctl = False
toc_title = ''
preface_title = ''
incdir = ''
toc_list = []
section_brk = '<!--section_break-->'    #SECTION BREAKER
page_brk = '<!--page_break-->'          #CHAPTER BREAKER
subdir = 'OEBPS'


def usage(progname,htmlfile,modfile):
    print """\
Usage: %s [-showvar] [-i INC_FILE.py] %s [%s]""" % (progname,htmlfile,modfile,)
    print """
Convert one html file to epub.
"""
    print """\
Parameters:
    -showvar: show internal variables that can modify for properly convert.
        example: %s -showvar > INC_FILE.py.""" % (progname,) + """
    -i: include additional variables from INC_FILE.py.
"""
    print """\
Other parameters in INC_FILE.py:
    use: %s -showvar
        to display, or
    use: %s -showvar > INC_FILE.py
        to edit and used.
""" % (progname,progname,)
    print """\
To do:
    - ISBN supported, currently just use UUID.
"""

# ---------- EPUB 2.0 FORMAT --------------------
#- META-INF/
#    - container.xml
#- mimetype
#- OPS/
#    - chap_xxxx.html
#    - ...
#    - content.opf
#    - cover.jpg
#    - css/
#        - page_styles.css
#        - stylesheet.css
#    - images/
#    - titlepage.xhtml
#    - toc.ncx
#
#----- container.xml ----------
#<?xml version="1.0"?>
#<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
#   <rootfiles>
#      <rootfile full-path="OPS/content.opf" media-type="application/oebps-package+xml"/>
#   </rootfiles>
#</container>
#
#----- mimetype ----------
#application/epub+zip
#
#----- content.opf ----------
#<?xml version='1.0' encoding='utf-8'?>
#<package xmlns="http://www.idpf.org/2007/opf" version="2.0" unique-identifier="uuid_id">
#  <metadata xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:opf="http://www.idpf.org/2007/opf" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:dc="http://purl.org/dc/elements/1.1/">
#    <dc:language>th</dc:language>
#    <dc:title>BookTitle</dc:title>
#    <dc:creator opf:role="aut">BookAuthor</dc:creator>
#    <dc:identifier id="uuid_id" opf:scheme="uuid">2057652c-0d00-4a4d-afc7-11171b95138b</dc:identifier>
#    <meta name="cover" content="titleimg"/>
#  </metadata>
#  <manifest>
#    <item href="titlepage.xhtml" id="cover" media-type="application/xhtml+xml" />
#    ...
#  </manifest>
#  <spine toc="ncx">
#    <itemref idref="cover" />
#    ...
#  </spine>
#  <guide>
#    <reference type="cover" href="titlepage.xhtml" />
#  </guide>
#</package>
#
#----- titlepage.xhtml ----------
#<?xml version='1.0' encoding='utf-8'?>
#<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
#<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
#<head>
#  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
#  <meta name="calibre:cover" content="true"/>
#  <title>Cover</title>
#  <style type="text/css" title="override_css">
#    @page {padding: 0pt; margin:0pt}
#    body { text-align: center; padding:0pt; margin: 0pt; }
#  </style>
#</head>
#<body>
#  <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" width="100%" height="100%" viewBox="0 0 536 751" preserveAspectRatio="none">
#    <image width="536" height="751" xlink:href="cover.jpg" />
#  </svg>
#</body>
#</html>
#
#----- toc.ncx ----------
#<?xml version='1.0' encoding='utf-8'?>
#<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="th-TH">
#  <head>
#    <meta name="dtb:uid" content="2057652c-0d00-4a4d-afc7-11171b95138b" />
#    <meta name="dtb:depth" content="2" />
#    <meta name="dtb:totalPageCount" content="0" />
#    <meta name="dtb:maxPageNumber" content="0" />
#  </head>
#  <docTitle>
#    <text>BookTitle</text>
#  </docTitle>
#  <navMap>
#    <navPoint id="navPoint-1" playOrder="1">
#      <navLabel>
#        <text>BookTitle</text>
#      </navLabel>
#      <content src="chap_0000.html" />
#    ...
#    </navPoint>
#  </navMap>
#</ncx>
#


# ---------- EPUB 2.0.1 FORMAT --------------------
#- META-INF/
#    - container.xml
#- mimetype
#- OEBPS/
#    - chap_xxxx.xhtml
#    - ...
#    - css/
#        - page_styles.css
#        - stylesheet.css
#    - content.opf
#    - cover.jpg
#    - images/
#    - titlepage.xhtml
#    - toc.ncx
#
#----- container.xml ----------
#<?xml version="1.0"?>
#<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
#   <rootfiles>
#      <rootfile full-path="OPS/content.opf" media-type="application/oebps-package+xml"/>
#   </rootfiles>
#</container>
#
#----- mimetype ----------
#application/epub+zip
#
#----- content.opf ----------
#<?xml version='1.0' encoding='utf-8'?>
#<package xmlns="http://www.idpf.org/2007/opf" version="2.0" unique-identifier="uuid_id">
#  <metadata xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:opf="http://www.idpf.org/2007/opf" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:dc="http://purl.org/dc/elements/1.1/">
#    <dc:language>th</dc:language>
#    <dc:title>BookTitle</dc:title>
#    <dc:creator opf:role="aut">BookAuthor</dc:creator>
#    <dc:identifier id="uuid_id" opf:scheme="uuid">2057652c-0d00-4a4d-afc7-11171b95138b</dc:identifier>
#    <meta name="cover" content="titleimg"/>
#  </metadata>
#  <manifest>
#    <item href="titlepage.xhtml" id="cover" media-type="application/xhtml+xml" />
#    ...
#  </manifest>
#  <spine toc="ncx">
#    <itemref idref="cover" />
#    ...
#  </spine>
#  <guide>
#    <reference type="cover" href="titlepage.xhtml" />
#  </guide>
#</package>
#
#----- titlepage.xhtml ----------
#<?xml version='1.0' encoding='utf-8'?>
#<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
#<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
#<head>
#  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
#  <meta name="calibre:cover" content="true"/>
#  <title>Cover</title>
#  <style type="text/css" title="override_css">
#    @page {padding: 0pt; margin:0pt}
#    body { text-align: center; padding:0pt; margin: 0pt; }
#  </style>
#</head>
#<body>
#  <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" width="100%" height="100%" viewBox="0 0 536 751" preserveAspectRatio="none">
#    <image width="536" height="751" xlink:href="cover.jpg" />
#  </svg>
#</body>
#</html>
#
#----- toc.ncx ----------
#<?xml version='1.0' encoding='utf-8'?>
#<!DOCTYPE ncx PUBLIC "-//NISO//DTD ncx 2005-1//EN" "http://www.daisy.org/z3986/2005/ncx-2005-1.dtd">
#<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="th-TH">
#  <head>
#    <meta name="dtb:uid" content="2057652c-0d00-4a4d-afc7-11171b95138b" />
#    <meta name="dtb:depth" content="2" />
#    <meta name="dtb:totalPageCount" content="0" />
#    <meta name="dtb:maxPageNumber" content="0" />
#  </head>
#  <docTitle>
#    <text>BookTitle</text>
#  </docTitle>
#  <navMap>
#    <navPoint id="navPoint-1" playOrder="1">
#      <navLabel>
#        <text>BookTitle</text>
#      </navLabel>
#      <content src="chap_0000.html" />
#    ...
#    </navPoint>
#  </navMap>
#</ncx>
#


# ---------- EPUB 3.0 FORMAT --------------------
#- META-INF/
#    - container.xml
#- mimetype
#- OEBPS/
#    - chap_xxxx.xhtml
#    - ...
#    - css/
#        - page_styles.css
#        - stylesheet.css
#    - content.opf
#    - cover.jpg
#    - images/
#    - titlepage.xhtml
#    - toc.ncx  (optional)
#
#----- container.xml ----------
#<?xml version="1.0"?>
#<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
#   <rootfiles>
#      <rootfile full-path="OPS/content.opf" media-type="application/oebps-package+xml"/>
#   </rootfiles>
#</container>
#
#----- mimetype ----------
#application/epub+zip
#
#----- content.opf ----------
#<?xml version="1.0" encoding="UTF-8" ?>
#<package xmlns="http://www.idpf.org/2007/opf" version="3.0" unique-identifier="uuid_id" xml:lang="th-TH">
#  <metadata xmlns:dc="http://purl.org/dc/elements/1.1/">
#    <dc:language>th</dc:language>
#    <dc:title>BookTitle</dc:title>
#    <dc:creator opf:role="aut">BookAuthor</dc:creator>
#    <dc:identifier id="uuid_id" opf:scheme="uuid">2057652c-0d00-4a4d-afc7-11171b95138b</dc:identifier>
#    <meta name="cover" content="titleimg"/>
#  </metadata>
#  <manifest>
#    <item href="titlepage.xhtml" id="cover" media-type="application/xhtml+xml" />
#    ...
#    <item href="chap_toc.xhtml" id="html_id_toc" properties="nav" media-type="application/xhtml+xml" />
#    ...
#    <!-- ncx included for 2.0 reading system compatibility: -->
#    <item href="toc.ncx" media-type="application/x-dtbncx+xml" id="ncx" />
#    ...
#  </manifest>
#  <spine toc="ncx">
#    <itemref idref="cover" />
#    ...
#  </spine>
#</package>
#
#----- titlepage.xhtml ----------
#<?xml version='1.0' encoding='utf-8'?>
#<!DOCTYPE html>
#<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
#<head>
#  <meta http-equiv="default-style" content="text/html; charset=UTF-8"/>
#  <meta name="calibre:cover" content="true"/>
#  <title>Cover</title>
#  <style type="text/css" title="override_css">
#    @page {padding: 0pt; margin:0pt}
#    body { text-align: center; padding:0pt; margin: 0pt; }
#  </style>
#</head>
#<body>
#  <section>
#    <img width="536" height="751" src="cover.jpg" />
#  </section>
#</body>
#</html>
#
#----- toc.ncx (OPTIONAL) ----------
#<?xml version='1.0' encoding='utf-8'?>
#<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="th-TH">
#  <head>
#    <meta name="dtb:uid" content="2057652c-0d00-4a4d-afc7-11171b95138b" />
#    <meta name="dtb:depth" content="2" />
#    <meta name="dtb:totalPageCount" content="0" />
#    <meta name="dtb:maxPageNumber" content="0" />
#  </head>
#  <docTitle>
#    <text>BookTitle</text>
#  </docTitle>
#  <navMap>
#    <navPoint id="navPoint-1" playOrder="1">
#      <navLabel>
#        <text>BookTitle</text>
#      </navLabel>
#      <content src="chap_0000.html" />
#    ...
#    </navPoint>
#  </navMap>
#</ncx>
#

style_file = ""
language = 'THAI'
lang = 'th'
full_lang = 'th-TH'
add_toc_brk_list = []   #[ [TAG,LEVEL],...].
#add_toc_brk_list = [
#    ['<p class="ph1"',1],
#    ['<p class="ph2"',2],
#]
mod_toc_title_list = [] #[ [CHAP_FILE#ID,LABEL,LEVEL],...]
#mod_toc_title_list = [
#    ["chap_0002.html#ch_0002_2_1","Change To Title1",1],
#    ["chap_0004.html#ch_0004_2_1","Change To Title2",1],
#]

#SHOW VARIABLE
#   [TYPE,ACTION, [ [NAME,DESC],[...] ] ],...
showvar_list = [\
    ["int","replace",[
        ["max_toc_level", "max level of TOC, [1..6], default is %s" % (max_toc_level,)],
        ]],
    ["string","replace",[
        ["epub_ver", "make EPUB version, %s, default is %s" % (epub_ver_list,epub_ver,)],
        ["language", "language to be used, ['ENGLISH','THAI'], default is 'THAI')."],
        ["style_file","stylesheet file to replace original in html file."],
        ["coverimg", "cover image file."],
        ["incdir", "include directory (will copy all files/dir in this dir into %s/)." % (subdir,)],
        ["book_uuid", "UUID book identifier."],
        ["book_isbn", "ISBN book identifier."],
        ]],
    ["string","append",[
        ]],
    ["list","append",[
        ["add_toc_brk_list","additional tags to be use for breaking chapter: [ [TAG,LEVEL],...]."],
        ["mod_toc_title_list","text to override default toc header and toc level: [ [CHAP_FILE#ID,LABEL,LEVEL],...] (use LEVEL=0 is no modified)."],
        ]],
    ["logical","replace",[
        ["ctl", "convert html tag to lowercase, running slow, not necessary if tag is already lower."],
        ]],
    ]

def showvar(progname):
    sep_list_1 = ['"',"'",'"""',"'''"]
    sep_list_2 = ['"""',"'''"]
    print '''\
#!/usr/bin/env python
# coding=utf-8
# ---------- %s SECTION ----------
#
# variable that can be overriden.''' % (progname,)
    print '''\

#VARIABLE ORDER: no order required.

'''
    for vtype,vmethod,varlist in showvar_list:
        print "#-----type: %s, will %s default variable of same name-----" % (vtype.upper(),vmethod.upper(),)
        for var,desc in varlist:
            print "#%s : %s" % (var,desc,)
            tmp = '%s' % (eval(var),)
            if vtype == "string":
                tmp = tmp.replace('\\','\\\\')
                if '\n' in tmp:
                    sep_list = sep_list_2
                else:
                    sep_list = sep_list_1
                for sep in sep_list:
                    if sep not in tmp:
                        break
                print '%s = %s%s%s' % (var,sep,tmp,sep,)
            else:   #LIST
                print '%s = %s' % (var,tmp,)
            print
        print


def cannotopenfile(filename):
    print "Cannot open file %s" % (filename)

def get_or_ignore_file(filename,msg):
    c = filename
    if os.path.isfile(filename):
        return filename
    else:
        print "msg" % (filename,)
        return ""

def get_or_ignore_dir(dirname,msg):
    c = dirname
    if os.path.isdir(dirname):
        return dirname
    else:
        print "msg" % (dirname,)
        return ""

def genbakfile(filename="",ext="bak"):
    if filename=="":
        return ""
    if os.path.exists(filename+"."+ext):
        i=0
        while os.path.exists(filename+"."+ext+str(i)) and (i < 1000):
            i += 1
        if i>999:
            return ""
        return filename+"."+ext+str(i)
    else:
        return filename+"."+ext

def tag2lower(txt):
    st = ['</','<']
    et = [' ','>']
    nmax = 999999
    front = ''
    rear = txt
    while rear and '<' in rear:
        tag = ''
        n = nmax
        for i in st:
            if i in rear:
                pos = rear.index(i)
                if pos<n:
                    n = pos
                    tag = i
        if n<nmax:
            tmp,rear = rear.split(tag,1)
            front += tmp+tag
        else:
            return front+rear
        n = nmax
        if rear and rear[0].isalpha():
            for i in et:
                if i in rear:
                    pos = rear.index(i)
                    if pos<n:
                        n = pos
                        tag = i
        if n<nmax:
            tmp,rear = rear.split(tag,1)
            front += tmp.lower() + tag
    return front+rear

def get_html_tag(t):
    #'<p class="xxx">' -> ['<p ','<p>','</p>']
    if '<' in t:
        t = t.split('<',1)[1]
    if ' ' in t:
        t = t.split(' ',1)[0]
    if '>' in t:
        t = t.split('>',1)[0]
    #return [beg_tag1, beg_tag2, end_tag]
    if '<!' in t and '<!--' not in t:
        return ['<%s ' % (t,), '<%s>' % (t,), '>' ]
    elif '<!--' in t:
        return ['<%s ' % (t,), '<%s>' % (t,), '-->' ]
    else:
        return ['<%s ' % (t,), '<%s>' % (t,), '</%s>' % (t,)]

def get_content_from_tag(txt,tag):
    tb1,tb2,te1 = get_html_tag(tag)
    rear = txt
    mid = ''
    if tag in rear:
        tmp,rear = rear.split(tag,1)
        if '>' not in tag:
            tmp,rear = rear.split('>',1)
        if te1 in rear:
            mid,rear = rear.split(te1,1)
            while mid.count(tb1)+mid.count(tb2) != mid.count(te1):
                mid += te1
                tmp,rear = rear.split(te1,1)
                mid += tmp
        else:
            mid = rear
            rear = ''
        return mid
    else:
        return ''

def get_content_from_block(txt,begblk,endblk):
    rear = txt
    mid = ''
    if begblk in rear:
        tmp,rear = rear.split(begblk,1)
        if endblk in rear:
            mid,rear = rear.split(endblk,1)
        else:
            mid = rear
            rear = ''
        return mid
    else:
        return ''

#rep_sim_list = [\
#    ['src','dst'],
#]
def replace_simple(txt,rl=[]):
    for i,j in rl:
        if i and i in txt:
            if i in j:
                txt = txt.replace(i,j)
            else:
                while i in txt:
                    txt = txt.replace(i,j)
    return txt

#rep_between_tag_list = [\
#       [bet_beg,bet_end, [[s1,d1],[s2,d2],[...]]],
#    ]
#example:   [\
#       ['<p class="vocab">','</p>',[ ['<b>','['], ['</b>',']'] ]],
#   ]
def replace_between_tag(txt,rl=[]):
    front = mid = ''
    rear = txt
    for i in rl:
        while i[0] in rear:
            beg1,beg2,end1 = get_html_tag(i[0])
            mid,rear = rear.split(i[0],1)
            front += mid
            mid,rear = rear.split(i[1],1)
            tmp = mid
            while beg1 in tmp or beg2 in tmp:
                tmp,rear = rear.split(i[1],1)
                mid += i[1]+tmp
            #for j in i[2]:
            #    mid = mid.replace(j[0],j[1])
            #front += i[0]+mid+i[1]
            #USE WHOLE BLOCK, INCLUDE BEGIN AND END TAG
            mid = i[0]+mid+i[1]
            for j in i[2]:
                mid = mid.replace(j[0],j[1])
            front += mid
        rear = front+rear
        front = ''
    return rear


#strip_tag_keep_txt_list = [ 'BEG_TAG1','BEG_TAG2',... ]
def strip_tag_keep_text(txt,rl=[]):   #['<dl class="DEMO1">','<!DOCTYPE',...]
    front = mid = ''
    rear = txt
    for tag in rl:
        begtag,tt,endtag = get_html_tag(tag)
        while tag in rear:
            tmp,rear = rear.split(tag,1)
            front += tmp
            if endtag != '>' and '>' not in tag and '>' in rear:
                tmp,rear = rear.split('>',1)
            if endtag == '>':
                tmp,rear = rear.split('>',1)
            else:
                if endtag in rear:
                    tmp,rear = rear.split(endtag,1)
                    mid = tmp
                    while rear and mid.count(begtag) != mid.count(endtag):
                        mid += endtag
                        tmp,rear = rear.split(endtag,1)
                        mid += tmp
                    front += mid
                else:
                    front += rear
                    rear = ''
        rear = front+rear
        front = ''
    return rear


def get_footnote(txt):
    #GREP footnote_list FROM content AND RETURN content,footnote_list
    #footnote_list = [ [HREF,TXT],... ]
    out_tag_list = [['<hr class="footnote"','>'],]
    keep_tag_list = [['<p class="footnote"','</p>'],['<div class="footnote"','</div>'],]
    all_tag_list = out_tag_list + keep_tag_list
    ot_list = [i for i,j in out_tag_list]
    kt_list = [i for i,j in keep_tag_list]
    at_list = ot_list + kt_list
    footnote_list = []
    front = ''
    rear = txt
    nmax = 99999999
    while rear:
        n = nmax
        btag = ''
        for i in at_list:
            if i in rear:
                ndx = rear.index(i)
                if ndx < n:
                    n = ndx
                    btag = i
        if btag:
            etag = all_tag_list[at_list.index(btag)][1]
            tmp,rear = rear.split(btag,1)
            front += tmp
            if etag in rear:
                tmp,rear = rear.split(etag,1)
                if btag in kt_list:
                    href = get_content_from_block(tmp,'href="#','"')    #HREF OF FOOTNOTE=ID OF SOURCE
                    tmp = btag + tmp + etag
                    footnote_list.append([href,tmp])
            else:
                front += btag + rear   #error
                rear = ''
        else:
            front += rear
            rear = ''
    return front,footnote_list
            
def insert_footnote_into_content(content_list,footnote_list):
    hrtxt = '<hr class="footnote" />'
    for href,txt in footnote_list:
        aid = ' id="%s"' % (href,)
        for i in range(len(content_list)):
            if aid in content_list[i]:
                if hrtxt in content_list[i]:
                    content_list[i] += '\n\n%s' % (txt,)
                else:
                    content_list[i] += '\n\n%s\n\n%s' % (hrtxt,txt,)
    return True

def get_chap_ext():
    if epub_ver == '2.0':
        return 'html'
    else:
        return 'xhtml'

def get_chap_filename(chap):
    return 'chap_%04d.%s' % (chap,get_chap_ext(),)


# *** DO BOTH ***
#   - INSERT HEADING ID IN CONTENT LIST (REMOVE OLD IF EXIST)
#   - RETURN TOC LIST    
#content_list = [ HTML1,HTML2,... ]
#toc_list = [ [HTML_FILE#ID, TITLE, LEVEL],... ]
def insert_heading_id_and_get_toc(content_list):

    def insert_id(txt,cid):
        #'<h1>' -> '<h1'+''+'>':                        txt = ''            -> ' id="X"'
        #'<h1 class="x">' -> '<h1'+' class="x"'+'>':    txt = ' class="x"'  -> ' id="X" class="x"'
        #'<h1 id="x">' -> '<h1'+' id="x"'+'>':          txt = ' id="x"'     -> ' id="X"'
        if 'id=' in txt:
            tmp,txt = txt.split('id=')
            while txt and txt[0]==' ':
                txt = txt[1:]
            if txt:
                c = txt[0]
                txt = txt[1:].split(c,1)[1]
            return tmp + 'id="%s"' % (cid,) + txt
        else:
            return ' id="%s"' % (cid,) + txt

    toc_list = []
    ext = get_chap_ext()
    for i in range(len(content_list)):
        front = ''
        rear = content_list[i]
        hl = []
        occ = {}
        ispass = False
        while rear:
            min_pos = 999999
            min_tag = ''
            min_lvl = 0
            for tag,lvl in toc_brk_list:
                if tag in rear:
                    pos = rear.index(tag)
                    if pos < min_pos:
                        min_tag = tag
                        min_lvl = lvl
                        min_pos = pos
            if min_tag:
                if min_tag in occ.keys():
                    occ[min_tag] += 1
                else:
                    occ[min_tag] = 1
                heading = get_content_from_tag(rear,min_tag)
                hl.append([heading,min_lvl,occ[min_tag]])
                tmp,rear = rear.split(min_tag,1)
                front += tmp+min_tag
                id_txt = 'ch_%04d_%s_%s' % (i,min_lvl,occ[min_tag],)
                if '>' in rear:
                    tmp,rear = rear.split('>',1)
                    front += insert_id(tmp,id_txt)+'>'
                else:
                    front += rear
                    rear = ''
                if not ispass:  #FIRST OCCUR
                    toc_list.append(['chap_%04d.%s' % (i,ext,),heading,min_lvl])
                else:
                    toc_list.append(['chap_%04d.%s#%s' % (i,ext,id_txt,),heading,min_lvl])
                ispass = True
            else:
                front += rear
                rear = ''
        if not ispass:
            heading = 'Title'
            toc_list.append(['chap_%04d.%s' % (i,ext,),heading,1])

        content_list[i] = front+rear

    return toc_list

def get_title_page(titleimg):
    txt = ''
    if epub_ver in ['2.0','2.0.1']:
        txt += '''\
<?xml version='1.0' encoding='utf-8'?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  <meta name="calibre:cover" content="true"/>
  <title>Cover</title>
  <style type="text/css" title="override_css">
    @page {padding: 0pt; margin:0pt}
    body { text-align: center; padding:0pt; margin: 0pt; }
  </style>
</head>
<body>
  <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" width="100%" height="100%" viewBox="0 0 536 751" preserveAspectRatio="none">
    <image width="536" height="751" ''' + '''xlink:href="%s" />''' % (titleimg,) + '''
  </svg>
</body>
</html>'''
    elif epub_ver in ['3.0']:
        txt += '''\
<?xml version='1.0' encoding='utf-8'?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
  <meta http-equiv="default-style" content="text/html; charset=UTF-8"/>
  <meta name="calibre:cover" content="true"/>
  <title>Cover</title>
  <style type="text/css" title="override_css">
    @page {padding: 0pt; margin:0pt}
    body { text-align: center; padding:0pt; margin: 0pt; }
  </style>
</head>
<body>
  <section>
    <img width="536" height="751" ''' + '''src="%s" />''' % (titleimg,) + '''
  </section>
</body>
</html>'''
    return txt


def get_container_xml(subdir):
    f = 'content.opf'
    if subdir:
        fn = os.path.join(subdir,f)
    else:
        fn = f
    return '''\
<?xml version="1.0" encoding="UTF-8" ?>
<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
   <rootfiles>
      <rootfile full-path="%s" media-type="application/oebps-package+xml" />
   </rootfiles>
</container>''' % (fn,)


def get_content_opf(title,author,content_list,titleimg="",other_files=[]):
    #other_files = [ [HREF,ID,MEDIA-TYPE], ... ]
    txt = ''
    if epub_ver in ['2.0','2.0.1']:
        txt += '''\
<?xml version='1.0' encoding='utf-8' ?>
<package xmlns="http://www.idpf.org/2007/opf" version="2.0" unique-identifier="uuid_id">
  <metadata xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:opf="http://www.idpf.org/2007/opf" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:dc="http://purl.org/dc/elements/1.1/">
    <dc:title>%s</dc:title>
    <dc:language>%s</dc:language>
    <dc:creator opf:role="aut">%s</dc:creator>
    <dc:identifier id="uuid_id" opf:scheme="uuid">%s</dc:identifier>
''' % (title,lang,author,book_uuid,)

    elif epub_ver in ['3.0']:
        txt += '''\
<?xml version="1.0" encoding="UTF-8" ?>
<package xmlns="http://www.idpf.org/2007/opf" version="3.0" unique-identifier="uuid_id" xml:lang="%s">
  <metadata xmlns:dc="http://purl.org/dc/elements/1.1/">
    <dc:title>%s</dc:title>
    <dc:language>%s</dc:language>
    <dc:creator>%s</dc:creator>
    <dc:identifier id="uuid_id">%s</dc:identifier>
    <meta property="dcterms:modified">%s</meta>
''' % (full_lang,title,lang,author,book_uuid,datetime.datetime.now().isoformat().split('.',1)[0]+'Z',)


    if titleimg:
        txt += '''\
    <meta name="cover" content="titleimg" />
'''
    txt += '''\
  </metadata>
  <manifest>
'''

    #TITLE IMAGE
    if titleimg:
        txt += '''\
    <item href="%s" id="%s" media-type="application/xhtml+xml" />
''' % ('titlepage.xhtml','cover',)
        txt += '''\
    <item href="%s" id="%s" media-type="image/jpeg" />
''' % (titleimg,'titleimg',)
        
    #OTHER FILES; TTF
    if other_files:
        for ohref,oid,omt in other_files:
            txt += '''\
    <item href="%s" id="%s" media-type="%s" />
''' % (ohref,oid,omt,)

    #CONTENT FROM TOC
    for i in range(len(content_list)):
        href = get_chap_filename(i)
        txt += '''\
    <item href="%s" id="html_id_%04d" media-type="application/xhtml+xml" />
''' % (href,i,)

    #ADD chap_toc.html
    if epub_ver in ['2.0','2.0.1']:
        txt += '''\
    <item href="%s" id="html_id_toc" media-type="application/xhtml+xml" />
''' % ('chap_toc.'+get_chap_ext())
    elif epub_ver in ['3.0']:
        txt += '''\
    <item href="%s" id="html_id_toc" properties="nav" media-type="application/xhtml+xml" />
''' % ('chap_toc.'+get_chap_ext())

    txt += '''\
    <item href="css/page_styles.css" id="page_css" media-type="text/css" />
    <item href="css/stylesheet.css" id="css" media-type="text/css" />
'''
    if epub_ver in ['3.0']:
        txt += '''\
    <!-- ncx included for 2.0 reading system compatibility: -->
'''

    txt += '''\
    <item href="toc.ncx" media-type="application/x-dtbncx+xml" id="ncx" />
  </manifest>
  <spine toc="ncx">
'''

    if titleimg:
        txt += '''\
    <itemref idref="%s" />
''' % ('cover',)

    for i in range(len(content_list)):
        txt += '''\
    <itemref idref="html_id_%04d" />
''' % (i,)

    txt += '''\
  </spine>
'''
    if epub_ver in ['2.0','2.0.1']:
        if titleimg:
            txt += '''\
  <guide>
    <reference type="cover" href="%s" />
  </guide>
''' % ('titlepage.xhtml',)
        else:
            txt += '''\
  <guide />
'''

    txt += '''\
</package>
'''
    return txt

def clean_br_span(txt):
    #clean tag <br>,<span lang="pi"> from title
    for i in ['<br />','<br>']: #CLEAN BR
        txt = txt.replace(i,' ')
    for i in ['&#173;']:       #CLEAN &HYP: FOR GOOGLE PLAY BOOKS
        txt = txt.replace(i,'')
    txt = strip_tag_keep_text(txt,rl=['<span','<b','<i','<a'])   #CLEAN <span>,<b>,<i>,<a>
    return txt


def get_toc_ncx(title,author,toc_list):
    txt = ''
    if epub_ver in ['2.0','3.0']:
        txt += '''\
<?xml version="1.0" encoding="UTF-8"?>
'''
    else:   #['2.0.1']
        txt += '''\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE ncx PUBLIC "-//NISO//DTD ncx 2005-1//EN" "http://www.daisy.org/z3986/2005/ncx-2005-1.dtd">
'''
    txt += '''\
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="%s">
  <head>
    <meta name="dtb:uid" content="%s" />
    <meta name="dtb:depth" content="%s" />
    <meta name="dtb:totalPageCount" content="0" />
    <meta name="dtb:maxPageNumber" content="0" />
  </head>
  <docTitle>
    <text>%s</text>
  </docTitle>
  <docAuthor>
    <text>%s</text>
  </docAuthor>
  <navMap>
''' %  (full_lang,book_uuid,max_toc_level,clean_br_span(title),clean_br_span(author),)

    spc = '  '
    level_use = prev_level = prev_level_use = 1  #USE level_use TO PREVENT STEP 1->3->2 ERROR
    diff = 0
    #toc_list = [ [CHAP_HTML#ID,HEADING,LEVEL],...]
    is_start = True
    for i in range(len(toc_list)):
        chap_id, label, level = toc_list[i]
        if level < 1:   #error
            level = 1
        #PREVENT MISS STEP
        level_use = prev_level_use + level - prev_level
        if level_use-prev_level_use > 1:
            level_use = prev_level_use+1
        if level == 1 and level != level_use:
            level_use = 1
        if level_use == 1 and level != 1:
            level_use = 2

        #CLOSE PREV
        if is_start:
            is_start = False
        else:
            for j in range(prev_level_use,level_use-1,-1):
                txt += '''\
%s</navPoint>
''' % (spc*(j+1),)
        #CURRENT
        s = spc*(level_use+1)
        txt += '''\
%s<navPoint id="navPoint-%s" playOrder="%s">
%s  <navLabel>
%s    <text>%s</text>
%s  </navLabel>
%s  <content src="%s" />
''' % (s,i+1,i+1, s, s,clean_br_span(label), s, s,chap_id,)
        prev_level = level
        prev_level_use = level_use

    #CLOSE
    for j in range(level_use,0,-1):
        txt += '''\
%s</navPoint>
''' % (spc*(j+1),)

    txt += '''\
  </navMap>
</ncx>
'''
    return txt


def get_toc_html(toc_list):
    """Rebuild TOC html from toc_list var."""
    #toc_list = [ [CHAP_HTML#ID,HEADING,LEVEL],...]
    txt = ''
    if epub_ver in ['2.0','2.0.1']:
        txt = '''\
<div class="_epubtoc">
'''
    elif epub_ver in ['3.0']:
        txt = '''\
<nav epub:type="toc" id="toc">
'''

    txt += '''\
<h2>%s</h2>
''' % (toc_title,)
    spc = '  '
    #START
    txt += '''\
  <ol>
'''
    last_level = 1
    for i in range(len(toc_list)):
        if toc_list[i][1] != toc_title:
            chap_id = toc_list[i][0]
            label = toc_list[i][1]
            level = toc_list[i][2]
            if i<(len(toc_list)-1):
                next_level = toc_list[i+1][2]
            else:
                next_level = 1
            if next_level > level:                  #NEST TOC
                #OPEN LIST SPAN
                txt += '''\
%s<li><span><a href="%s">%s</a></span>
%s<ol>
''' % (spc*(level+1),chap_id,label,spc*(level+1),)
                if epub_ver in ['2.0','2.0.1']:
                    for j in range(level+1,next_level): #MISS STEP NEST TOC LEVEL
                        txt += '''\
%s<li>
%s<ol>
''' % (spc*(j+1),spc*(j+1),)
                elif epub_ver in ['3.0']:
                    for j in range(level+1,next_level): #MISS STEP NEST TOC LEVEL
                        txt += '''\
%s<li><span class="_displaynone">-</span>
%s<ol>
''' % (spc*(j+1),spc*(j+1),)
            else:   # next_level <= level:
                #NORMAL LIST
                txt += '''\
%s<li><a href="%s">%s</a></li>
''' % (spc*(level+1),chap_id,label,)
            if next_level < level:   #next_level < level
                #CLOSE LIST SPAN
                for j in range(level,next_level,-1):    #CLOSE NEST
                    txt += '''\
%s</ol>
%s</li>
''' % (spc*j,spc*j,)
        old_level = level

    #END
    txt += '''\
  </ol>
'''
    if epub_ver in ['2.0','2.0.1']:
        txt += '''\
</div>
'''
    if epub_ver in ['3.0']:
        txt += '''\
</nav>
'''
    return txt


def get_content_html(content_txt,toc_list):
    #for tag in ['<h1','<h2','<h3','<h4','___tag___']:
    for tag,lvl in toc_brk_list + [['___tag___',0]]:
        if tag in content_txt:
            break
    if tag != '___tag___':
        title = get_content_from_tag(content_txt,tag)
    else:
        title = 'Title'
    title = clean_br_span(title)

    txt = ''
    if epub_ver == '2.0':
        txt += '''\
<?xml version='1.0' encoding='utf-8'?>
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
  <title>%s</title>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
  <link href="css/stylesheet.css" type="text/css" rel="stylesheet"/>
  <link href="css/page_styles.css" type="text/css" rel="stylesheet"/>
</head>
<body>

%s

</body>
</html>
''' % (title,content_txt,)
    elif epub_ver == '2.0.1':
        txt += '''\
<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="%s">
<head>
  <title>%s</title>
  <meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />
  <link href="css/stylesheet.css" type="text/css" rel="stylesheet"/>
  <link href="css/page_styles.css" type="text/css" rel="stylesheet"/>
</head>
<body>

%s

</body>
</html>
''' % (lang,title,content_txt,)
    elif epub_ver == '3.0':
        txt += '''\
<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="%s" lang="%s" xmlns:epub="http://www.idpf.org/2007/ops">
<head>
  <title>%s</title>
  <meta http-equiv="default-style" content="application/xhtml+xml; charset=utf-8" />
  <link href="css/stylesheet.css" type="text/css" rel="stylesheet" />
  <link href="css/page_styles.css" type="text/css" rel="stylesheet" />
</head>
<body>

%s

</body>
</html>
''' % (lang,lang,title,content_txt,)
    return txt

known_mimetype = {
#    'zip':  'application/epub+zip',
    'css':  'text/css',
    'gif':  'image/gif',
    'jpg':  'image/jpeg',
    'jpeg': 'image/jpeg',
    'png':  'image/png',
    'svg':  'image/svg+xml',
    'otf':  'application/x-font-opentype',
    'ttf':  'application/x-font-truetype',
    }
known_mimetype_upper = [{i.upper():j} for i,j in known_mimetype.iteritems()]

def get_mimetype(txt):
    if txt in known_mimetype.keys():
        return known_mimetype[txt]
    elif txt in known_mimetype_upper.keys():
        return known_mimetype_upper[txt]
    return 'unknown'


def get_other_files(txt):
    #GET other_files FOR content.opf
    other_files = []    #[ [HREF,ID,MEDIA-TYPE], ... ]

    # FROM STYLESHEET
    if style_file:
        for font in ['otf','ttf']:
            tmp = commands.getstatusoutput("sed '/\\/\\*/,/\\*\\//d' %s | grep url | grep \".%s\"" % (style_file,font,))
            #(USE sed TO EXCLUDE COMMENT /* ... */)
            omt = get_mimetype(font)
            if tmp[1]:
                for i in tmp[1].split('\n'):
                    ohref = i.split('(',1)[1].split(')',1)[0]
                    oid = ohref.split('/')[-1]
                    other_files.append([ohref,oid,omt])
    
    # FROM SELF HTML
    # IMG
    front = ""
    rear = txt
    btag = '<img'
    etag = '>'
    while btag in rear:
        t = get_content_from_block(rear,btag,etag) 
        ohref = t.split('src=',1)[1].split(' ',1)[0].strip('"').strip("'")
        oid = ohref.split('/')[-1]
        for it in known_mimetype.keys() + ['__unknown___']:
            if '.'+it in ohref:
                break
        omt = get_mimetype(it)
        other_files.append([ohref,oid,omt])
        rear = rear.split(btag,1)[1]
    return other_files

def make_epub(rootdir,epub_filename,title,author,content_list,toc_list,toc_in_content_ndx):
    tmpl = os.path.realpath(os.curdir).split(os.path.sep)
    if not epub_filename:
        epub_filename = '-'.join([tmpl[-2],tmpl[-1]]) + '.epub'
    if os.path.isdir(rootdir):
        shutil.rmtree(rootdir)
    os.mkdir(rootdir)

    #META-INF
    fd = os.path.join(rootdir,'META-INF')
    if not os.path.isdir(fd):
        os.mkdir(fd)

    #META-INF/container.xml
    fn = os.path.join(rootdir,'META-INF','container.xml')
    open(fn,'w').write(get_container_xml(subdir))

    #MIMETYPE
    fn = os.path.join(rootdir,'mimetype')
    open(fn,'w').write('application/epub+zip')
    
    dirname = os.path.join(rootdir,subdir)
    os.mkdir(dirname)

    #TITLE PAGE
    if coverimg:
        titleimg = 'cover.jpg'
        shutil.copy2(coverimg,os.path.join(dirname,titleimg))
        fn = os.path.join(dirname,'titlepage.xhtml')
        open(fn,'w').write(get_title_page(titleimg))
    else:
        titleimg = ''

    sf = style_file
    if not os.path.isfile(sf):
        cannotopenfile(sf+' (Warn)')
        sf = ""
    if sf:
        stylesheet = open(sf).read()
    else:
        stylesheet = get_content_from_tag(txt,'<style>')

    if epub_ver in ['2.0','2.0.1']: #ADD TOC DISPLAY SUPPORT
        stylesheet += '''
._epubtoc ol {
  list-style-type: none;
}
'''
    elif epub_ver in ['3.0']:
        stylesheet += '''
._displaynone {
  display: none;
}
'''

    #STYLESHEET
    dn = os.path.join(dirname,'css')
    if not os.path.isdir(dn):
        os.mkdir(dn)
    fn = os.path.join(dn,'stylesheet.css')
    open(fn,'w').write(stylesheet)
    #PAGE STYLESHEET
    fn = os.path.join(dn,'page_styles.css')
    open(fn,'w').write('''\
@page {
    margin-bottom: 5pt;
    margin-top: 5pt
    }
''')

    #GET other_files FOR MAKE content.opf AND MAKE SUBDIR IN ROOT
    other_files = get_other_files(txt)

    #content.opf
    fn = os.path.join(dirname,'content.opf')
    #toc_list = [ [CHAP_NO,"LABEL","ID",LEVEL],...]
    open(fn,'w').write(get_content_opf(title,author,content_list,titleimg,other_files=other_files))

    #toc.ncx
    fn = os.path.join(dirname,'toc.ncx')
    open(fn,'w').write(get_toc_ncx(title,author,toc_list))

    #chap_nnnn.html
    for i in range(len(content_list)):
        fn = os.path.join(dirname,get_chap_filename(i))
        open(fn,'w').write(get_content_html(content_list[i],toc_list))

    #MAKE chap_toc.html FOR USE WITH EPUB3
    #toc_list = [ [CHAP_HTML#ID,HEADING,LEVEL],...]
    if toc_in_content_ndx < 0:
        tmp_txt = get_toc_html(toc_list)
    else:
        tmp_txt = content_list[toc_in_content_ndx]
    fn = os.path.join(dirname,'chap_toc.'+get_chap_ext())
    open(fn,'w').write(get_content_html(tmp_txt,toc_list))

    #include directory
    if incdir and os.path.isdir(incdir):
        distutils.dir_util.copy_tree(incdir,dirname)

    #MAKE EPUB
    if os.path.isfile(epub_filename):
        os.remove(epub_filename)

    os.system('''\
cd %s; zip -qX0 ../%s mimetype; zip -qrX9 ../%s * -x mimetype; cd - \
''' % (rootdir, epub_filename, epub_filename,))
    return True

def convert_html(txt,epub_filename=""):
    if ctl:
        txt = tag2lower(txt)

    global toc_title,style_file

    title = get_content_from_tag(txt,'<title>')
    if not title:
        title = 'Title'
    author = get_content_from_block(txt,'<meta name="author" content="','" />')
    if not author:
        author = 'Author'

    #MAKE content_list
    #content_list = [ HTML1, HTML2, HTML3, ... ]
    content = get_content_from_tag(txt,'<body')

    #PREPARE CONTENT FOR EPUB
    rep_sim_list = [
        ['<wbr>',   '&#8204;'],
        ['<wbr />', '&#8204;'],
        ['&shy;',   '&#173;'],
        ['<br>',    '<br />'],
        ['<u>',     '<span class="underline">'],
        ['</u>',    '</span>'],
    ]
    content = replace_simple(content,rl=rep_sim_list)

    #GREP FOOTNOTE section FROM content
    #footnote_list = [ [HREF,TXT],... ]
    content,footnote_list = get_footnote(content)

    if epub_ver in ['3.0']:
        #CLEAN <span> FROM HEADING, CONVERT TAG TO xml
        #rep_between_tag_list = [\
        #       [bet_beg,bet_end, [[d1,i1],[d2,i2],[...]]],
        #    ]
        rep_between_tag_list = []
        for i in ['h1','h2','h3','h4','h5','h6']:
            rep_between_tag_list.append(['<%s' % (i,),'</%s>' % (i,), \
                [['<span lang="pi">',''],['</span>','']] \
            ])
        rep_between_tag_list.append(['<span','>', \
            [[' lang=',' xml:lang=']] \
        ])
        replace_between_tag(content,rl=rep_between_tag_list)

    #TRY TO BREAK CONTENT WITH <!--section_break> OR <!--page_break-->
    if section_brk in txt or page_brk in txt:
        content_list = [content]
        nlc = []
        for i in [section_brk,page_brk]:
            for j in content_list:
                if i in j:
                    nlc.extend(j.split(i))
                else:
                    nlc.append(j)
            content_list = nlc[:]
            nlc = []
    else:   #BREAK FROM LEAST LEVEL HEADING
        #for sep in ['<h1','<h2','<h3','<h4','___sep___']:
        for sep,lvl in toc_brk_list + [['___sep___',0]]:
            if sep in content:
                break
        if sep != '___sep___':
            content_list = content.split(sep)
            for i in range(len(content_list))-1:
                content_list[i+1] = sep+content_list[i+1]
        else:
            content_list = [content]


    #INSERT HEADING ID IN content_list AND RETURN toc_list
    #toc_list = [ [CHAP_HTML#ID,HEADING,LEVEL],...]
    toc_list = insert_heading_id_and_get_toc(content_list)

    #INSERT FOOTNOTE BACK INTO PROPER CONTENT
    tmp = insert_footnote_into_content(content_list,footnote_list)

    #MODIFY CHAPTER TITLE
    #mod_toc_title_list = [ [CHAPTER#ID,LABEL,LEVEL],... ]
    mod_toc_chap_list = [x for x,y,z in mod_toc_title_list]
    mod_toc_label_list = [y for x,y,z in mod_toc_title_list]
    mod_toc_level_list = [z for x,y,z in mod_toc_title_list]
    #toc_list = [ [CHAP_NO,"LABEL","ID",LEVEL],...]
    for i in range(len(toc_list)):
        if toc_list[i][0] in mod_toc_chap_list:
            ndx = mod_toc_chap_list.index(toc_list[i][0])
            if toc_list[i][0] == mod_toc_chap_list[ndx]:
                toc_list[i][1] = mod_toc_label_list[ndx]
                if mod_toc_level_list[ndx] != 0:    #MOD LEVEL = 0 = NO MOD
                    toc_list[i][2] = mod_toc_level_list[ndx]

    #IF THERE IS TOC TITLE IN HTML HEADING, WE WILL REBUILD TOC CHAPTER AND REPLACE IT BACK
    #toc_list = [ [CHAP_HTML#ID,HEADING,LEVEL],...]
    toc_in_content_ndx = -1
    tl1 = [j for i,j,k in toc_list]
    if toc_title in tl1:
        #REBUILD TOC HTML
        ndx = tl1.index(toc_title)
        toc_in_content_ndx = int(toc_list[ndx][0].split('chap_',1)[1].split('.',1)[0])
        content_list[toc_in_content_ndx] = get_toc_html(toc_list)    

    #START MAKE EPUB
    rootdir = 'epubdir'
    make_epub(rootdir,epub_filename,title,author,content_list,toc_list,toc_in_content_ndx)

    return ''


if __name__=="__main__":
    progname=os.path.basename(sys.argv[0])
    if '-showvar' in sys.argv or '-showvars' in sys.argv:
        showvar(progname)
        sys.exit(0)

    if '-ctl' in sys.argv:
        sys.argv.pop(sys.argv.index('-ctl'))
        ctl = True

    if '-i' in sys.argv:
        ndx = sys.argv.index('-i')
        sys.argv.pop(ndx)
        tmp_incpath = sys.argv.pop(ndx)
        tmp_incfile = os.path.basename(tmp_incpath)
        tmp_incdir = os.path.dirname(tmp_incpath)
        try:
            curdir = os.path.realpath(os.path.curdir)
            sys.path.append(os.path.abspath(os.path.join(curdir,tmp_incdir)))   # SET PATH TO IMPORT FILE DIR.
        except:
            #BUG OF SHELL getcwd() IN MOUNT SYSTEM
            curdir = commands.getoutput('ls -l /proc/self/cwd | cut -d\> -f2 | cut -d\  -f2')
            sys.path.append(os.path.abspath(os.path.join(curdir,tmp_incdir)))   # SET PATH TO IMPORT FILE DIR.

        impfile = os.path.splitext(tmp_incfile)[0]
        # IMPORT FILE
        incadd = __import__(impfile)
        try:    #EPUB VERSION
            c = incadd.epub_ver
            if c in epub_ver_list:
                epub_ver = c
            else:
                print "'%s' not in %s, use default '%s'." % (c,epub_ver_list,epub_ver,)
        except:
            pass
        try:    #LANGUAGE
            c = incadd.language.upper()
            language  = c
        except:
            pass
        try:    #STYLESHEET FILE
            c = incadd.style_file
            style_file = get_or_ignore_file(c,"style_file:%s, file not found, ignore.")
        except:
            pass
        try:    #COVER IMAGE
            c = incadd.coverimg
            coverimg = get_or_ignore_file(c,"coverimg:%s, file not found, ignore.")
        except:
            pass
        try:    #INCLUDE DIRECTORY
            c = incadd.incdir
            incdir = get_or_ignore_dir(c,"incdir:%s, directory not found, ignore.")
        except:
            pass
        try:    #TOC LEVEL
            n = incadd.max_toc_level
            if n>0 and n<7:
                max_toc_level = n
            else:
                print "%s: value not between 1 and 6, use default '%s'." % (n,max_toc_level,)
        except:
            pass
        try:    #UUID
            c = incadd.book_uuid
            book_uuid  = c
        except:
            pass
        try:    #ISBN
            c = incadd.book_isbn
            book_isbn = c
        except:
            pass
        try:    #TOC BREAK LIST
            toc_brk_list.extend(incadd.add_toc_brk_list)
        except:
            pass
        try:    #MODIFY TOC TITLE LIST
            mod_toc_title_list.extend(incadd.mod_toc_title_list)
            mod_toc_title_list = list(set(mod_toc_title_list))
        except:
            pass

    if language[:3].upper() == 'ENG':
        lang = 'en'
        full_lang = 'en-US'
        toc_title = 'Table of Contents'
        preface_title = 'Preface'
    else:   #DEFAULT THAI
        lang = 'th'
        full_lang = 'th-TH'
        toc_title = 'สารบัญ'
        preface_title = 'คำนำ'

    try:
        htmlfile = sys.argv[1]
    except:
        usage(progname,'FILE.html','FILE.epub')
        sys.exit(1)
    try:
        modfile = sys.argv[2]
    except:
        modfile = os.path.splitext(os.path.basename(htmlfile))[0]+'.epub'
    try:
        fsold=open(htmlfile)
        txt = fsold.read()
        fsold.close()
    except:
        cannotopenfile(htmlfile)
        usage(progname,'FILE.html','FILE.epub')
        sys.exit(1)

    bakfile = ""
    #if os.path.exists(modfile):
    #    bakfile = genbakfile(modfile,'bak')

    if bakfile:
        os.rename(modfile,bakfile)

    #REDUCE TOC BREAK LIST TO MATCH MAX NUMBER OF TOC LEVEL
    toc_brk_list = [[i,j] for i,j in toc_brk_list if j<= max_toc_level]
    toc_brk_list = [[i.split('>',1)[0],j] for i,j in toc_brk_list]
    #SORT (USE FOR BREAKING CHAPTER)
    tmp_list = [[j,i] for i,j in toc_brk_list]
    tmp_list.sort()
    toc_brk_list = [[j,i] for i,j in tmp_list]

    #MAKE EPUB
    convert_html(txt,modfile)

    filetype = 'epub'
    if bakfile:
        print "Convert html to %s in %s to %s success, save backup file in %s" % (filetype,htmlfile,modfile,bakfile)
    else:
        print "Convert html to %s in %s to %s success." % (filetype,htmlfile,modfile)
